@@ -1,6 +1,6 @@
 #!/bin/bash
 
-set -eu
+set -euo pipefail
 
 if [[ "${BIFROST_TRACE:-}" == true ]]; then
     set -x
@@ -8,11 +8,20 @@ fi
 
 declare -A PKG_MAP
 
+# NOTE(rpittau): we need a stable recent version of pip to avoid issues with
+# the cryptography package.
+PIP_MIN_REQ="20.0.1"
+PIP_TUPLE="(20, 0, 1)"
+
 # workaround: for latest bindep to work, it needs to use en_US local
 export LANG=en_US.UTF-8
 
 export VENV=${VENV:-/opt/stack/bifrost}
 
+# NOTE(dtantsur): change this when creating a stable branch
+BRANCH=${ZUUL_BRANCH:-victoria}
+CONSTRAINTS_FILE=${UPPER_CONSTRAINTS_FILE:-${TOX_CONSTRAINTS_FILE:-https://releases.openstack.org/constraints/upper/$BRANCH}}
+
 CHECK_CMD_PKGS=(
     python3-devel
     python3
@@ -64,7 +73,7 @@ case ${ID,,} in
 
     rhel|fedora|centos)
     OS_FAMILY="RedHat"
-    PKG_MANAGER=$(which dnf || which yum)
+    PKG_MANAGER=$(/usr/bin/which dnf || /usr/bin/which yum)
     if [[ "${BIFROST_TRACE:-}" != true ]]; then
         PKG_MANAGER="$PKG_MANAGER --quiet"
     fi
@@ -82,7 +91,7 @@ case ${ID,,} in
     *) echo "ERROR: Supported package manager not found.  Supported: apt, dnf, yum, zypper"; exit 1;;
 esac
 
-echo Installing Python and PIP
+echo "Installing Python and PIP"
 
 for pkg in ${CHECK_CMD_PKGS[@]}; do
     if ! $(${CHECK_CMD} ${PKG_MAP[$pkg]} &>/dev/null); then
@@ -129,11 +138,12 @@ if [[ "${BIFROST_TRACE:-}" != true ]]; then
     PIP="$PIP --quiet"
 fi
 
-$PYTHON << EOF
-import pip
-version = tuple(map(int, pip.__version__.split('.')))
-assert version >= (7, 1)
-EOF
+# NOTE(rpittau): we need a stable recent version of pip to avoid issues with
+# the cryptography package.
+PIP_REQUIRED=$($PYTHON -c "import pip; print(tuple(map(int, pip.__version__.split('.'))) >= $PIP_TUPLE)")
+if [[ $PIP_REQUIRED == "False" ]]; then
+    ${PIP} install "pip==$PIP_MIN_REQ"
+fi
 
 export PIP_OPTS="--upgrade-strategy only-if-needed"
 
@@ -145,6 +155,6 @@ echo "Using Bindep to install binary dependencies"
 bindep -b &> /dev/null || ${INSTALLER_CMD} $(bindep -b)
 
 echo "Installing Python requirements"
-${PIP} install -r "$(dirname $0)/../requirements.txt" -c ${UPPER_CONSTRAINTS_FILE:-https://releases.openstack.org/constraints/upper/victoria}
+${PIP} install -r "$(dirname $0)/../requirements.txt" -c "${CONSTRAINTS_FILE}"
 
 echo "Completed installation of basic dependencies."
